/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.mistakesanderrors.errorinlife;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * A simple example, demonstrating J-Sim's possibilities to catch exception thrown in the life() method of your process. An
 * ArithmeticException (division by zero) is used here as example.
 * 
 * In such a case, J-Sim destroys your process but keeps other processes running.
 */
public class ErrorInLife
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		BadDivisionProcess process = null;

		try
		{
			// First, we create a simulation and one process.
			simulation = new JSimSimulation("Error In Life Simulation");
			process = new BadDivisionProcess("Bad Guy", simulation);

			// We must not forget to activate the process. A nice time, isn't it?
			process.activate(1.2345);

			// Now we run the simulation. We will do as many steps as it is needed by all processes present in the system.
			// This is potentially dangerous if there is at least one process that never wants to finish.
			while (simulation.step() == true)
				;
		} // try

		// All possible exceptions are caught here in one handler.
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch

		// It is a good practice to put the shutdown in the finally block. Then you can be sure that the JVM will not hang.
		finally
		{
			simulation.shutdown();
		}
	} // main

} // class ErrorInLife

/**
 * A simple process example. It divides number 100 by ten different numbers and crashes at 100/0. Note that we must use integer division.
 * Dividing a float number by zero returns Infinity which is not an error.
 * 
 */
class BadDivisionProcess extends JSimProcess
{
	/**
	 * Note that you must ALWAYS write your own constructor &ndash; JSimProcess' one has two parameters.
	 */
	public BadDivisionProcess(String name, JSimSimulation sim) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
	} // constructor

	/**
	 * This is the life of the process. We will cycle here ten times, dividing numbers.
	 */
	protected void life()
	{
		try
		{
			int i;

			message("Hello! I have just been born!");

			for (i = 10; i >= 0; i--)
			{
				message("Hello!");
				message("100 divided by " + i + " is " + 100 / i);
				// This hold() is not necessary for this demonstration.
				hold(9.8765);
			} // for i

			message("You should never see this message.");
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
	} // life

} // class BadDivisionProcess
