/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.mistakesanderrors.forbiddenpassivate;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/*
 *  A simple example, demonstrating that a process may passivate only itself, not another one.
 *
 */
public class ForbiddenPassivate
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		BadPassivateProcess badProcess = null, goodProcess = null;
		int i;

		try
		{
			// First, we create a simulation and two processes.
			simulation = new JSimSimulation("Forbidden Passivate Simulation");
			goodProcess = new BadPassivateProcess("Good Guy", simulation, null);
			badProcess = new BadPassivateProcess("Bad Guy", simulation, goodProcess);

			// We must not forget to activate both processes.
			goodProcess.activate(1.0);
			badProcess.activate(2.0);

			// Now we do just five steps. It is enough to show the feature.
			for (i = 0; (i < 5) && (simulation.step() == true); i++)
				;
		} // try

		// All three possible exceptions are caught here in one handler.
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
		
		// It is a good practice to put the shutdown in the finally block. Then you can be sure that the JVM will not hang.
		finally
		{
			simulation.shutdown();
		} // finally
	} // main

} // class ForbiddenPassivate

/**
 * A simple process example. It tries to passivate another process but is not successful.
 */ 
class BadPassivateProcess extends JSimProcess
{
	private BadPassivateProcess procToPassivate;

	/**
	 * Note that you must ALWAYS write your own constructor &ndash; JSimProcess' one has two parameters.
	 */ 
	public BadPassivateProcess(String name, JSimSimulation sim, BadPassivateProcess proc) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
		procToPassivate = proc;
	} // constructor

	/**
	 * This is the life of the process. We will try to passivate the other guy or print hello.
	 */ 
	protected void life()
	{
		try
		{
			// Are we good or bad?
			if (procToPassivate != null) // Bad
			{
				message(getName() + ": Hello! I'm going to call " + procToPassivate.getName() + "'s passivate() method!");
				procToPassivate.passivate();
				message(getName() + ": It's done, bye.");
			} // if
			else
				// Good
				while (true)
				{
					message(getName() + ": Hello! I'm still here!");
					hold(0.9);
				} // while
		} // try
		catch (JSimException e)
		{
			error(getName() + ": I'm so sorry! I will never do this again.");
			e.printStackTrace();
			e.printComment();
		} // catch
	} // life

} // class BadPassivateProcess
