/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.systemutils.meanvalues;

import cz.zcu.fav.kiv.jsim.JSimSystem;

/**
 * An example on generating random values with different distributions. Mean values of generated numbers are calculated and compared to the
 * theoretical ones.
 * 
 * Please note that this example takes a very long time to run.
 */
public class MeanValuesExample
{
	private static final int NUM_OF_TURNS = 1000000;

	public static void main(String[] args)
	{
		// Testing the generator of exponential distribution
		exponMeanValue(0.25);
		exponMeanValue(0.40);
		exponMeanValue(0.50);
		exponMeanValue(0.75);
		exponMeanValue(1.00);
		exponMeanValue(2.00);
		exponMeanValue(2.50);
		exponMeanValue(5.00);
		exponMeanValue(0.00); // Should return 0.0, although theoretically it is infinity

		// Testing the generator of uniform distribution
		uniformMeanValue(0.0, 10.0);
		uniformMeanValue(-50.0, +50.0);
		uniformMeanValue(-15.0, +30.0);
		uniformMeanValue(Double.MIN_VALUE, Double.MAX_VALUE); // Returns infinity because of sumation
		uniformMeanValue(99.0, 100.0);
		uniformMeanValue(0.0, 0.0);

		// Testing the generator of normal (Gauss) distribution
		normalMeanValue(5.0, 1.0);
		normalMeanValue(50.0, 50.0);
		normalMeanValue(0.0, 1000.0);
		normalMeanValue(-2.33, 67.555);
		normalMeanValue(560.0, 0.0);
		normalMeanValue(0.0, 0.0);

		// Testing the generator of boolean values
		probabilityMeanValue(0.0);
		probabilityMeanValue(0.25);
		probabilityMeanValue(0.50);
		probabilityMeanValue(0.75);
		probabilityMeanValue(1.0);
		probabilityMeanValue(1000.0); // Should return 1.0
		probabilityMeanValue(-5.0); // Should return 0.0
	} // main

	private static void exponMeanValue(double lambda)
	{
		int i;
		double sum = 0.0;

		System.out.println("Wait, please, generating " + NUM_OF_TURNS + " random numbers with exponential distribution...");

		for (i = 0; i < NUM_OF_TURNS; i++)
			sum += JSimSystem.negExp(lambda);

		System.out.println("Lambda = " + lambda + "; Theoretical m.v. = " + (1 / lambda) + "; Obtained m.v. = " + (sum / NUM_OF_TURNS));
	} // exponMeanValue

	private static void uniformMeanValue(double a, double b)
	{
		int i;
		double sum = 0.0;

		System.out.println("Wait, please, generating " + NUM_OF_TURNS + " random numbers with uniform distribution...");

		for (i = 0; i < NUM_OF_TURNS; i++)
			sum += JSimSystem.uniform(a, b);

		System.out.println("A = " + a + "; B = " + b + "; Theoretical m.v. = " + ((a + b) / 2) + "; Obtained m.v. = " + (sum / NUM_OF_TURNS));
	} // uniformMeanValue

	private static void normalMeanValue(double mu, double sigma)
	{
		int i;
		double sum = 0.0;

		System.out.println("Wait, please, generating " + NUM_OF_TURNS + " random numbers with normal distribution...");

		for (i = 0; i < NUM_OF_TURNS; i++)
			sum += JSimSystem.gauss(mu, sigma);

		System.out.println("Mu = " + mu + "; Sigma = " + sigma + "; Theoretical m.v. = " + mu + "; Obtained m.v. = " + (sum / NUM_OF_TURNS));
	} // normalMeanValue

	private static void probabilityMeanValue(double p)
	{
		int i;
		int sum = 0;

		System.out.println("Wait, please, generating " + NUM_OF_TURNS + " boolean values...");

		for (i = 0; i < NUM_OF_TURNS; i++)
			if (JSimSystem.draw(p) == true)
				sum++;

		System.out.println("P = " + p + "; Theoretical m.v. = " + Math.max(Math.min(p, 1.0), 0.0) + "; Obtained m.v. = " + ((double) sum / (double) NUM_OF_TURNS));
	} // normalMeanValue

} // class MeanValuesExample
