/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.systemutils.randomstreams;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.random.JSimDrawStream;
import cz.zcu.fav.kiv.jsim.random.JSimExponentialStream;
import cz.zcu.fav.kiv.jsim.random.JSimGaussianStream;
import cz.zcu.fav.kiv.jsim.random.JSimUniformStream;

/**
 * An example on generating random values with different distributions using streams from the Random subpackage. Mean values of generated
 * numbers are calculated and compared to the theoretical ones.
 * 
 * Please note that this example takes a very long time to run.
 */
public class MeanValuesExample
{
	private static final int NUM_OF_TURNS = 1000000;

	public static void main(String[] args)
	{
		// Testing the generator of exponential distribution.
		exponentialDistributionMeanValue(0.25);
		exponentialDistributionMeanValue(0.40);
		exponentialDistributionMeanValue(0.50);
		exponentialDistributionMeanValue(0.75);
		exponentialDistributionMeanValue(1.00);
		exponentialDistributionMeanValue(2.00);
		exponentialDistributionMeanValue(2.50);
		exponentialDistributionMeanValue(5.00);
		exponentialDistributionMeanValue(0.00); // Should return 0.0, although theoretically it is infinity

		System.out.println("--------------------------");

		// Testing the generator of uniform distribution.
		uniformDistributionMeanValue(0.0, 10.0);
		uniformDistributionMeanValue(-50.0, +50.0);
		uniformDistributionMeanValue(-15.0, +30.0);
		uniformDistributionMeanValue(Double.MIN_VALUE, Double.MAX_VALUE); // Returns infinity because of sumation
		uniformDistributionMeanValue(99.0, 100.0);
		uniformDistributionMeanValue(0.0, 0.00001);

		System.out.println("--------------------------");

		// Testing the generator of normal (Gaussian) distribution.
		normalDistributionMeanValue(5.0, 1.0);
		normalDistributionMeanValue(50.0, 50.0);
		normalDistributionMeanValue(0.0, 1000.0);
		normalDistributionMeanValue(-2.33, 67.555);
		normalDistributionMeanValue(560.0, 0.0);
		normalDistributionMeanValue(0.0, 0.0);

		System.out.println("--------------------------");

		// Testing the generator of boolean values.
		booleanGeneratorMeanValue(0.0);
		booleanGeneratorMeanValue(0.25);
		booleanGeneratorMeanValue(0.50);
		booleanGeneratorMeanValue(0.75);
		booleanGeneratorMeanValue(1.0);
	} // main

	private static void exponentialDistributionMeanValue(double lambda)
	{
		int i;
		double sum = 0.0;
		JSimExponentialStream s;

		try
		{
			System.out.println("Wait, please, generating " + NUM_OF_TURNS + " random numbers with exponential distribution...");
			s = new JSimExponentialStream(lambda);
			for (i = 0; i < NUM_OF_TURNS; i++)
				sum += s.getNext();
			System.out.println("Lambda = " + lambda + "; Theoretical m.v. = " + (1 / lambda) + "; Obtained m.v. = " + (sum / NUM_OF_TURNS));
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // exponentialDistributionMeanValue

	private static void uniformDistributionMeanValue(double a, double b)
	{
		int i;
		double sum = 0.0;
		JSimUniformStream s;

		try
		{
			System.out.println("Wait, please, generating " + NUM_OF_TURNS + " random numbers with uniform distribution...");
			s = new JSimUniformStream(a, b);
			for (i = 0; i < NUM_OF_TURNS; i++)
				sum += s.getNext();
			System.out.println("A = " + a + "; B = " + b + "; Theoretical m.v. = " + ((a + b) / 2) + "; Obtained m.v. = " + (sum / NUM_OF_TURNS));
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // uniformDistributionMeanValue

	private static void normalDistributionMeanValue(double mu, double sigma)
	{
		int i;
		double sum = 0.0;
		JSimGaussianStream s;

		try
		{
			System.out.println("Wait, please, generating " + NUM_OF_TURNS + " random numbers with normal distribution...");
			s = new JSimGaussianStream(mu, sigma);
			for (i = 0; i < NUM_OF_TURNS; i++)
				sum += s.getNext();
			System.out.println("Mu = " + mu + "; Sigma = " + sigma + "; Theoretical m.v. = " + mu + "; Obtained m.v. = " + (sum / NUM_OF_TURNS));
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // normalDistributionMeanValue

	private static void booleanGeneratorMeanValue(double p)
	{
		int i;
		int sum = 0;
		JSimDrawStream s;

		try
		{
			System.out.println("Wait, please, generating " + NUM_OF_TURNS + " boolean values...");
			s = new JSimDrawStream(p);
			for (i = 0; i < NUM_OF_TURNS; i++)
				if (s.getNext() == true)
					sum++;
			System.out.println("P = " + p + "; Theoretical m.v. = " + Math.max(Math.min(p, 1.0), 0.0) + "; Obtained m.v. = " + ((double) sum / (double) NUM_OF_TURNS));
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // booleanGeneratorMeanValue

} // class MeanValuesExample
