/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.performancetesting.speed;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * The aim of this program is to measure the time needed to switch between the simulation and a process (and back). In J-Sim 0.1.2, the
 * switching mechanism was substantially improved, therefore the results should differ considerably when two different version (0.1.2 and
 * older 0.1.1) are used.
 * 
 */
public class Speed
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		TestingProcess[] processes;
		int i;
		long time1, time2, difference, hours, minutes, seconds, millis;

		final int NO_OF_PROCESSES = 500;

		try
		{
			System.out.println("---------------------------------");
			System.out.println("Testing " + JSimSystem.getVersion());
			System.out.println("Number of processes is " + NO_OF_PROCESSES);
			System.out.println("---------------------------------");
			System.getProperties().list(System.out);
			System.out.println("---------------------------------");

			// First, we create a simulation and processes.
			simulation = new JSimSimulation("Performance Testing Simulation");

			processes = new TestingProcess[NO_OF_PROCESSES];
			for (i = 0; i < NO_OF_PROCESSES; i++)
			{
				processes[i] = new TestingProcess("Process No. " + Integer.toString(i + 1), simulation);
				processes[i].activate(i * 0.001);
			} // i

			// Now we run the simulation. We will execute 10000 steps.
			// This should let every process run ten times.
			time1 = System.currentTimeMillis();
			for (i = 0; (i < 10000) && (simulation.step() == true); i++)
				;
			time2 = System.currentTimeMillis();

			difference = time2 - time1;
			hours = difference / 3600000;
			difference %= 3600000;
			minutes = difference / 60000;
			difference %= 60000;
			seconds = difference / 1000;
			millis = difference % 1000;

			System.out.println("Time needed for 10.000 steps: " + hours + " hours " + minutes + " minutes " + seconds + " seconds " + millis + " milliseconds.");
		} // try
	
		// All possible exceptions are caught here in one handler.
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
		
		// It is a good practice to put the shutdown in the finally block. Then you can be sure that that the JVM will not hang.
		// Every simulation must be shut down at the end of the program, even if an error occurs.
		finally
		{
			simulation.shutdown();
		} // finally
	} // main
	
} // class Speed

/**
 * A process does some "useful" work and then suspends itself for a given fixed time. We must not use any I/O function (including message()
 * and error()) to avoid distorsion of the result.
 */
class TestingProcess extends JSimProcess
{
	/**
	 * Note that you must ALWAYS write your own constructor &ndash; JSimProcess' one has two parameters.
	 */
	public TestingProcess(String name, JSimSimulation sim) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
	} // constructor

	/**
	 * This is the life of the process. We will cycle here infinite number of times, doing some simple calculations.
	 */
	protected void life()
	{
		@SuppressWarnings("unused") // Not really used, serves only to have some code (which is certainly eliminated by the compiler).
		int i;

		try
		{
			while (true)
			{
				i = 1 + 1;
				hold(2.0);
			} // for i
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
	} // life

} // class TestingProcess
